#!/bin/bash
#===============================================================================
#
#          FILE:  generate_LRA.sh
# 
#         USAGE:  ./generate_LRA.sh 
# 
#   DESCRIPTION:  
# 
#       OPTIONS:  ---
#  REQUIREMENTS:  ---
#          BUGS:  ---
#         NOTES:  ---
#        AUTHOR:   (), 
#       COMPANY:  
#       VERSION:  1.0
#       CREATED:  05/02/24 15:18:57 EET
#      REVISION:  ---
#===============================================================================

# load fdb tools and set respective ECCODES directories
load_mars

# load cdo
load_cdo

# set FDB configuration
FDB5_CONFIG_FILE=/pfs/lustrep3/projappl/project_465000454/experiments/a0hy/fdb/REGULARLL_grids/etc/fdb/config.yaml

# set output directory for grib files
gribdir="/pfs/lustrep3/scratch/project_465000454/kkeller/lab/replicability_mixed/grib"

# set output directory for regular lat/lon netcdf files
ncdir="/pfs/lustrep3/scratch/project_465000454/kkeller/lab/replicability_mixed/netcdf"

# set basic experiment parameters
start_date=19900101
end_date=20200101
expid=a0hy
nmembers=10

# set additional parameters
activity=HighResMIP
experiment=cont

# parameter ids
a2d_tprate=235055
a2d_2t=228004
a2d_msl=151
a3d_t=130
a3d_u=131
a3d_v=132
a3d_q=133
o2d_avg_tas=263101
o2d_avg_tos=263100
o2d_avg_siconc=263001

a2d_all_ids=($a2d_tprate $a2d_2t $a2d_msl)
a3d_all_ids=($a3d_t $a3d_u $a3d_v $a3d_q)
o2d_all_ids=($o2d_avg_tas $o2d_avg_tos $o2d_avg_siconc)

a2d_all_names=("tprate" "2t" "msl")
a3d_all_names=("t" "u" "v" "q")
o2d_all_names=("avg_tas" "avg_tos" "avg_siconc")

# create output directory
mkdir -p $gribdir

# iterate over 2D atm variables
levtype=sfc
stream=clmn
resolution=high
year_str=${start_date%????}/to/${end_date%????}
id=(${a2d_all_ids[@]})
var=(${a2d_all_names[@]})
month_str=1/2/3/4/5/6/7/8/9/10/11/12
for ((i=0;i<${#a2d_all_ids[@]};i++)); do
  for ((m=1;m<=$nmembers;m++)); do
    echo "---> ${m}:${var[$i]} <---"
    echo "retrieve,class=d1,type=fc,dataset=climate-dt,model=IFS-NEMO,activity=$activity,experiment=$experiment,generation=1,realization=$m,expver=$expid,stream=$stream,resolution=$resolution,year=$year_str,month=$month_str,levtype=$levtype,param=${id[$i]}" > req
    fdb-read req $gribdir/fc${m}.${levtype}.${var[$i]}.${id[$i]}.${start_date}.${end_date}.mon.grib
  done
done

# iterate over 3D atm variables
levtype=pl
stream=clmn
resolution=high
year_str=${start_date%????}/to/${end_date%????}
id=(${a3d_all_ids[@]})
var=(${a3d_all_names[@]})
month_str=1/2/3/4/5/6/7/8/9/10/11/12
levelist_str=1/5/10/20/30/50/70/100/150/200/250/300/400/500/600/700/850/925/1000
for ((i=0;i<${#a3d_all_ids[@]};i++)); do
  for ((m=1;m<=$nmembers;m++)); do
    echo "---> ${m}:${var[$i]} <---"
    echo "retrieve,class=d1,type=fc,dataset=climate-dt,model=IFS-NEMO,activity=$activity,experiment=$experiment,generation=1,realization=$m,expver=$expid,stream=$stream,resolution=$resolution,year=$year_str,month=$month_str,levtype=$levtype,levelist=$levelist_str,param=${id[$i]}" > req
    fdb-read req $gribdir/fc${m}.${levtype}.${var[$i]}.${id[$i]}.${start_date}.${end_date}.mon.grib
  done
done

# iterate over 2D oce variables
levtype=o2d
stream=clte
resolution=standard
date_str=${start_date}/to/${end_date}
id=(${o2d_all_ids[@]})
var=(${o2d_all_names[@]})
time_str=0000
for ((i=0;i<${#o2d_all_ids[@]};i++)); do
  for ((m=1;m<=$nmembers;m++)); do
    echo "---> ${m}:${var[$i]} <---"
    echo "retrieve,class=d1,type=fc,dataset=climate-dt,model=IFS-NEMO,activity=$activity,experiment=$experiment,generation=1,realization=$m,expver=$expid,stream=$stream,resolution=$resolution,date=$date_str,time=$time_str,levtype=$levtype,param=${id[$i]}" > req
    fdb-read req $gribdir/fc${m}.${levtype}.${var[$i]}.${id[$i]}.${start_date}.${end_date}.mon.grib
  done
done

# generate regular lat lon data in netcdf
for f in $(find $gribdir -type f -size +0M); do 
  fn=$ncdir/$(basename ${f%.*}.regLL.1x1.nc)
  cdo --eccodes -t ecmwf -f nc copy -remapcon,r360x180 $f $fn 
done

# cleanup
rm req
